{*******************************************************}
{                                                       }
{                 "Virtual Town" project                }
{                                                       }
{               Copyright (c) 2006 PAS             }
{                                                       }
{Authors: Alex Ponomarev <AlxPonom@mail.ru>             }
{                                                       }
{  This program is free software under the GPL (>= v2)  }
{ Read the file COPYING coming with project for details }
{*******************************************************}

unit unitImageDivider;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Buttons, ExtDlgs, StdCtrls;

const
  FragmentsFolder = 'Fragments';
  ResultSegmentsFolder = 'Segments';
  ProjectFileName = 'Project.xml';
  
type
  TImageFragment = class
  private
    Next: TImageFragment;
    BMP: TBitmap;
    OriginFileName: string;
    OfsX: integer;
    OfsY: integer;

    Constructor Create(const JpegFile: string; const pOfsX,pOfsY: integer);
    Destructor Destroy; override;
    procedure LoadFromJpegFile(const JpegFile: string);
    function IsPointInside(const X,Y: integer): boolean;
  end;

  TProject = class
  private
    Fragments: TImageFragment;
    ProjectFile: string;
    ContainerColor: TColor;
    SegmentSize: integer;

    Constructor Create;
    Destructor Destroy; override;
    procedure Clear;
    procedure LoadFromFile(const FileName: string);
    procedure SaveToFile(const FileName: string);
    procedure CreateNew(const ProjectFileName: string);
    function AddNewFragment(const FragmentFileName: string; const pOfsX,pOfsY: integer): TImageFragment;
    procedure DestroyFragment(const DestroyFragment: TImageFragment);
    procedure MoveUpFragment(const FragmentToMove: TImageFragment);
    procedure MoveDownFragment(const FragmentToMove: TImageFragment);
    function FragmentAtPoint(const X,Y: integer): TImageFragment;
    procedure GetMinMax(out MinX,MinY,MaxX,MaxY: integer);
    procedure NormalizeMinMaxBySegmentSize(var MinX,MinY,MaxX,MaxY: integer; out Level: integer);
    procedure DivideFragmentImageToSegments();
  end;

  TfmImageDivider = class(TForm)
    PaintBox: TPaintBox;
    pnlControl: TPanel;
    sbChangeContainerColor: TSpeedButton;
    ColorDialog: TColorDialog;
    OpenPictureDialog: TOpenPictureDialog;
    sbAddNewFragment: TSpeedButton;
    sbNewProject: TSpeedButton;
    sbDivideImage: TSpeedButton;
    Panel1: TPanel;
    sbDestroySelectedFragment: TSpeedButton;
    sbMoveUpSelectedFragment: TSpeedButton;
    sbMoveDownSelectedFragment: TSpeedButton;
    Label1: TLabel;
    edSegmentSize: TEdit;
    procedure sbChangeContainerColorClick(Sender: TObject);
    procedure sbAddNewFragmentClick(Sender: TObject);
    procedure PaintBoxPaint(Sender: TObject);
    procedure PaintBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBoxMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBoxMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure sbDivideImageClick(Sender: TObject);
    procedure sbDestroySelectedFragmentClick(Sender: TObject);
    procedure sbMoveUpSelectedFragmentClick(Sender: TObject);
    procedure sbMoveDownSelectedFragmentClick(Sender: TObject);
    procedure sbNewProjectClick(Sender: TObject);
  private
    { Private declarations }
    Project: TProject;
    Project_BMP: TBitmap;
    Project__Out_OfsX: integer;
    Project__Out_OfsY: integer;
    Mouse_LastPos: TPoint;
    MovingFragment: TImageFragment;
    SelectedFragment: TImageFragment;
    flProjectMoving: boolean;

    procedure Project_Paint;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent); override;
    Destructor Destroy; override;
  end;

var
  fmImageDivider: TfmImageDivider;

implementation
Uses
  IJL,
  JPEG,
  MSXML,
  unitDividingProcess;

{$R *.dfm}


{TImageFragment}
Constructor TImageFragment.Create(const JpegFile: string; const pOfsX,pOfsY: integer);
begin
Inherited Create;
Next:=nil;
BMP:=nil;
OriginFileName:='';
OfsX:=pOfsX;
OfsY:=pOfsY;
LoadFromJpegFile(JpegFile);
end;

Destructor TImageFragment.Destroy;
begin
BMP.Free;
Inherited;
end;

procedure TImageFragment.LoadFromJpegFile(const JpegFile: string);

  function GetBitmapFromJPEGStream(const JPEGStream: TMemoryStream; const Bitmap: TBitmap): boolean;
  var
    iWidth, iHeight, iNChannels: Integer;
    iDIBChannels: integer;
    iStatus: integer;
    jcprops: TJPEG_CORE_PROPERTIES;
    DIB: TDIBSection;
  begin
  Result:=False;
  iDIBchannels:=3;
  FillChar(jcprops,SizeOf(jcprops),0);
  iStatus:=ijlInit(@jcprops);
  if (iStatus = IJL_OK)
   then
    try
    jcprops.JPGBytes:=JPEGStream.Memory;
    jcprops.JPGSizeBytes:=JPEGStream.Size;
    iStatus:=ijlRead(@jcprops,IJL_JBUFF_READPARAMS);
    if (iStatus = IJL_OK)
     then begin
      iWidth:=jcprops.JPGWidth;
      iHeight:=jcprops.JPGHeight;
      iNChannels:=jcprops.JPGChannels;
      if (iNChannels = 3)
       then begin
        iDIBChannels:=iNChannels;
        Bitmap.Width:=iWidth;
        Bitmap.Height:=iHeight;
        FillChar(DIB,SizeOf(DIB),0);
        iStatus:=GetObject(Bitmap.Handle,SizeOf(DIB),@DIB);
        if (iStatus <> 0)
         then begin
          jcprops.DIBWidth:=iWidth;
          jcprops.DIBHeight:=-iHeight;
          jcprops.DIBChannels:=iDIBChannels;
          jcprops.DIBColor:=IJL_BGR;
          jcprops.DIBPadBytes:=((((iWidth*iDIBChannels)+3) div 4)*4)-(iWidth*iDIBChannels);
          jcprops.DIBBytes:=System.PByte(DIB.dsBm.bmBits);
          iStatus:=ijlRead(@jcprops,IJL_JBUFF_READWHOLEIMAGE);
          Result:=(iStatus >= 0);
          end;
        end;
      end;
    finally
    ijlFree(@jcprops);
    end;
  end;

var
  MS: TMemoryStream;
begin
MS:=TMemoryStream.Create;
try
MS.LoadFromFile(JpegFile);
//.
BMP.Free;
BMP:=TBitmap.Create;
BMP.HandleType:=bmDIB;
BMP.PixelFormat:=pf24bit;
if (NOT GetBitmapFromJPEGStream(MS, BMP)) then Raise Exception.Create('could not get bitmap from jpeg file'); //. =>
finally
MS.Destroy;
end;
//.
OriginFileName:=JpegFile;
end;

function TImageFragment.IsPointInside(const X,Y: integer): boolean;
begin
if (BMP = nil)
 then begin
  Result:=false;
  Exit; //. ->
  end;
Result:=(
                ((OfsX <= X) AND (X <= OfsX+BMP.Width)) AND
                ((OfsY <= Y) AND (Y <= OfsY+BMP.Height))
        );
end;


Constructor TProject.Create;
begin
Inherited Create;
Fragments:=nil;
ProjectFile:='';
ContainerColor:=clGray;
SegmentSize:=256;
end;

Destructor TProject.Destroy;
begin
if (ProjectFile <> '') then SaveToFile(ProjectFile);
Clear;
Inherited;
end;

procedure TProject.LoadFromFile(const FileName: string);
var
  Doc: IXMLDOMDocument;
  RootNode: IXMLDOMNode;
  I: integer;
  FragmentNode: IXMLDOMNode;
  Node: IXMLDOMNode;
  OrgFileName: string;
  OfsX: integer;
  OfsY: integer;
begin
Clear;
//.
Doc:=CoDomDocument.Create;
Doc.Set_Async(false);
Doc.Load(FileName);
RootNode:=Doc.documentElement.selectSingleNode('/ROOT');
for I:=0 to RootNode.childNodes.length-1 do begin
  FragmentNode:=RootNode.childNodes[I];
  Node:=FragmentNode.selectSingleNode('File');
  OrgFileName:=Node.nodeTypedValue;
  Node:=FragmentNode.selectSingleNode('OfsX');
  OfsX:=Node.nodeTypedValue;
  Node:=FragmentNode.selectSingleNode('OfsY');
  OfsY:=Node.nodeTypedValue;
  //.
  AddNewFragment(OrgFileName,OfsX,OfsY);
  end;
//.
ProjectFile:=FileName;
end;

procedure TProject.SaveToFile(const FileName: string);
var
  Doc: IXMLDOMDocument;
  PI: IXMLDOMProcessingInstruction;
  Root: IXMLDOMElement;
  FragmentNode: IXMLDOMElement;
  Node: IXMLDOMElement;
  Fragment: TImageFragment;
begin
Doc:=CoDomDocument.Create;
Doc.Set_Async(false);
PI:=Doc.createProcessingInstruction('xml', 'version=''1.0''');
Doc.insertBefore(PI, Doc.childNodes.Item[0]);
Root:=Doc.createElement('ROOT');
Root.setAttribute('xmlns:dt', 'urn:schemas-microsoft-com:datatypes');
Doc.documentElement:=Root;
//.
Fragment:=Fragments;
while (Fragment <> nil) do with Fragment do begin
  FragmentNode:=Doc.createElement('Fragment');
    Node:=Doc.createElement('File');
    Node.nodeTypedValue:=Fragment.OriginFileName;
    FragmentNode.appendChild(Node);
    Node:=Doc.createElement('OfsX');
    Node.nodeTypedValue:=Fragment.OfsX;
    FragmentNode.appendChild(Node);
    Node:=Doc.createElement('OfsY');
    Node.nodeTypedValue:=Fragment.OfsY;
    FragmentNode.appendChild(Node);
  Root.appendChild(FragmentNode);
  //. next fragment
  Fragment:=Fragment.Next;
  end;
Doc.Save(FileName);
end;

procedure TProject.Clear;
var
  DestroyFragment: TImageFragment;
begin
while (Fragments <> nil) do begin
  DestroyFragment:=Fragments;
  Fragments:=DestroyFragment.Next;
  //.
  DestroyFragment.Destroy();
  end;
end;

procedure TProject.CreateNew(const ProjectFileName: string);
begin
if (ProjectFile <> '') then SaveToFile(ProjectFile);
Clear;
//.
ProjectFile:=ProjectFileName;
end;

function TProject.AddNewFragment(const FragmentFileName: string; const pOfsX,pOfsY: integer): TImageFragment;
var
  ptrptrFragment: pointer;
begin
Result:=TImageFragment.Create(FragmentFileName,pOfsX,pOfsY);
//.
ptrptrFragment:=@Fragments;
while (Pointer(ptrptrFragment^) <> nil) do ptrptrFragment:=@TImageFragment(ptrptrFragment^).Next;
TImageFragment(ptrptrFragment^):=Result;
end;

procedure TProject.DestroyFragment(const DestroyFragment: TImageFragment);
var
  LastFragment,Fragment: TImageFragment;
begin
LastFragment:=nil;
Fragment:=Fragments;
while (Fragment <> nil) do begin
  if (Fragment = DestroyFragment)
   then begin
    if (LastFragment <> nil)
     then LastFragment.Next:=Fragment.Next
     else Fragments:=Fragment.Next;
    //.
    Fragment.Destroy;
    //.
    Exit; //. ->
    end;
  //.
  LastFragment:=Fragment;
  Fragment:=Fragment.Next;
  end;
end;

procedure TProject.MoveUpFragment(const FragmentToMove: TImageFragment);
var
  LastFragment,Fragment: TImageFragment;
  F: TImageFragment;
begin
LastFragment:=nil;
Fragment:=Fragments;
while (Fragment <> nil) do begin
  if (Fragment = FragmentToMove)
   then begin
    if (Fragment.Next = nil) then Exit; //. ->
    if (LastFragment <> nil)
     then LastFragment.Next:=Fragment.Next
     else Fragments:=Fragment.Next;
    //.
    F:=Fragment.Next;
    Fragment.Next:=F.Next;
    F.Next:=Fragment;
    //.
    Exit; //. ->
    end;
  //.
  LastFragment:=Fragment;
  Fragment:=Fragment.Next;
  end;
end;

procedure TProject.MoveDownFragment(const FragmentToMove: TImageFragment);
var
  LastLastFragment,LastFragment,Fragment: TImageFragment;
  F: TImageFragment;
begin
LastFragment:=nil;
LastLastFragment:=nil;
Fragment:=Fragments;
while (Fragment <> nil) do begin
  if (Fragment = FragmentToMove)
   then begin
    if (LastFragment = nil) then Exit; //. ->
    //.
    LastFragment.Next:=Fragment.Next;
    if (LastLastFragment <> nil)
     then LastLastFragment.Next:=Fragment
     else Fragments:=Fragment;
    Fragment.Next:=LastFragment;
    //.
    Exit; //. ->
    end;
  //.
  LastLastFragment:=LastFragment;
  LastFragment:=Fragment;
  Fragment:=Fragment.Next;
  end;
end;

function TProject.FragmentAtPoint(const X,Y: integer): TImageFragment;
var
  Fragment: TImageFragment;
begin
Result:=nil;
Fragment:=Fragments;
while (Fragment <> nil) do with Fragment do begin
  if (Fragment.IsPointInside(X,Y)) then Result:=Fragment;
  //. next fragment
  Fragment:=Fragment.Next;
  end;
end;

procedure TProject.GetMinMax(out MinX,MinY,MaxX,MaxY: integer);
var
  Fragment: TImageFragment;
begin
MinX:=MaxInt;
MinY:=MaxInt;
MaxX:=-MaxInt;
MaxY:=-MaxInt;
Fragment:=Fragments;
while (Fragment <> nil) do with Fragment do begin
  if (OfsX < MinX) then MinX:=OfsX;
  if (OfsY < MinY) then MinY:=OfsY;
  if ((OfsX+BMP.Width) > MaxX) then MaxX:=(OfsX+BMP.Width);
  if ((OfsY+BMP.Height)> MaxY) then MaxY:=(OfsY+BMP.Height);
  //. next fragment
  Fragment:=Fragment.Next;
  end;
end;

procedure TProject.NormalizeMinMaxBySegmentSize(var MinX,MinY,MaxX,MaxY: integer; out Level: integer);

  function GetSize(const IniSize: integer; out Level: integer): integer;
  begin
  Result:=SegmentSize;
  Level:=0;
  while (Result < IniSize) do begin
    Result:=(Result SHL 1);
    Inc(Level);
    end;
  end;

var
  D,Dr: integer;
  L,Lr: integer;
begin
Dr:=GetSize(MaxX-MinX,Lr);
D:=GetSize(MaxY-MinY,L);
if (D > Dr)
 then begin
  Dr:=D;
  Lr:=L;
  end;
MaxX:=MinX+Dr;
MaxY:=MinY+Dr;
Level:=Lr;
end;

procedure TProject.DivideFragmentImageToSegments();
var
  SegmentBMP: TBitmap;
  SegmentJI: TJPegImage;
  Fragment: TImageFragment;
  MinX,MinY,MaxX,MaxY: integer;
  Level: integer;
  Size: integer;
  DivCount: integer;
  X,Y: integer;
  ShiftX,ShiftY: integer;
  OutRect: TRect;
begin
GetMinMax(MinX,MinY,MaxX,MaxY);
NormalizeMinMaxBySegmentSize(MinX,MinY,MaxX,MaxY,Level);
Size:=MaxX-MinX;
DivCount:=(Size DIV SegmentSize);
SegmentBMP:=TBitmap.Create;
try
SegmentBMP.Width:=SegmentSize;
SegmentBMP.Height:=SegmentBMP.Width;
SegmentJI:=TJPegImage.Create;
try
with TfmDividingProcess.Create(nil) do
try
Show();
with SegmentBMP do
for Y:=0 to DivCount-1 do begin
  for X:=0 to DivCount-1 do begin
    Canvas.Pen.Color:=ContainerColor;
    Canvas.Brush.Color:=Canvas.Pen.Color;
    Canvas.Rectangle(0,0,Width,Height);
    //.
    Fragment:=Fragments;
    while (Fragment <> nil) do with Fragment do begin
      //.
      ShiftX:=(OfsX-MinX)-X*SegmentSize;
      ShiftY:=(OfsY-MinY)-Y*SegmentSize;
      //.
      with OutRect do begin
      Left:=ShiftX;
      Top:=ShiftY;
      Right:=Left+BMP.Width;
      Bottom:=Top+BMP.Height;
      end;
      if (OutRect.Left < 0) then OutRect.Left:=0;
      if (OutRect.Right > SegmentSize) then OutRect.Right:=SegmentSize;
      if (OutRect.Left < OutRect.Right)
       then begin
        if (OutRect.Top < 0) then OutRect.Top:=0;
        if (OutRect.Bottom > SegmentSize) then OutRect.Bottom:=SegmentSize;
        if (OutRect.Top < OutRect.Bottom)
         then begin
          Canvas.CopyRect(OutRect,BMP.Canvas,Rect(OutRect.Left-ShiftX,OutRect.Top-ShiftY,OutRect.Right-ShiftX,OutRect.Bottom-ShiftY));
          end;
        end;
      //. next fragment
      Fragment:=Fragment.Next;
      end;
    //.
    SegmentJI.Assign(SegmentBMP);
    SegmentJI.SaveToFile(ExtractFilePath(Application.ExeName)+'\'+ResultSegmentsFolder+'\'+'Segment_'+IntToStr(Level)+'_'+IntToStr(X)+'_'+IntToStr(Y)+'.jpg');
    end;
  pbProgress.Position:=Round(pbProgress.Max*Y/DivCount);
  Update();
  end;
finally
Destroy;
end;
finally
SegmentJI.Destroy;
end;
finally
SegmentBMP.Destroy;
end;
ShowMessage('Image successfully devided in segments(tiles).'#$0D#$0A'Check '+ResultSegmentsFolder+' folder.'#$0D#$0A'"Size power of 2" = '+IntToStr(Level));
end;


{TfmImageDivider}
Constructor TfmImageDivider.Create(AOwner: TComponent);
begin
Inherited Create(AOwner);
DoubleBuffered:=true;
Project:=TProject.Create;
if (FileExists(ExtractFilePath(Application.ExeName)+'\'+ProjectFileName))
 then Project.LoadFromFile(ExtractFilePath(Application.ExeName)+'\'+ProjectFileName)
 else Project.ProjectFile:=ExtractFilePath(Application.ExeName)+'\'+ProjectFileName;
Project_BMP:=TBitmap.Create;
Project__Out_OfsX:=0;
Project__Out_OfsY:=0;
Mouse_LastPos.X:=-1;
MovingFragment:=nil;
SelectedFragment:=nil;
flProjectMoving:=false;
edSegmentSize.Text:=IntToStr(Project.SegmentSize)+'x'+IntToStr(Project.SegmentSize);
end;

Destructor TfmImageDivider.Destroy;
begin
Project_BMP.Free;
Project.Free;
Inherited;
end;

procedure TfmImageDivider.Project_Paint;
var
  Fragment: TImageFragment;
  MinX,MinY,MaxX,MaxY: integer;
  Level: integer;
  ShiftX,ShiftY: integer;
  OutRect: TRect;
begin
Project_BMP.Width:=PaintBox.Width;
Project_BMP.Height:=PaintBox.Height;
with Project_BMP do begin
Canvas.Pen.Width:=1;
Canvas.Brush.Style:=bsSolid;
//.
Canvas.Pen.Color:=Color;
Canvas.Brush.Color:=Canvas.Pen.Color;
Canvas.Rectangle(0,0,Width,Height);
//.
Project.GetMinMax(MinX,MinY,MaxX,MaxY);
Project.NormalizeMinMaxBySegmentSize(MinX,MinY,MaxX,MaxY,Level);
Canvas.Pen.Color:=Project.ContainerColor;
Canvas.Brush.Color:=Canvas.Pen.Color;
Canvas.Rectangle(MinX+Project__Out_OfsX,MinY+Project__Out_OfsY,MaxX+Project__Out_OfsX,MaxY+Project__Out_OfsY);
//.
Fragment:=Project.Fragments;
while (Fragment <> nil) do with Fragment do begin
  //.
  ShiftX:=OfsX+Project__Out_OfsX;
  ShiftY:=OfsY+Project__Out_OfsY;
  //.
  with OutRect do begin
  Left:=ShiftX;
  Top:=ShiftY;
  Right:=Left+BMP.Width;
  Bottom:=Top+BMP.Height;
  end;
  if (OutRect.Left < 0) then OutRect.Left:=0;
  if (OutRect.Right > Width) then OutRect.Right:=Width;
  if (OutRect.Left < OutRect.Right)
   then begin
    if (OutRect.Top < 0) then OutRect.Top:=0;
    if (OutRect.Bottom > Height) then OutRect.Bottom:=Height;
    if (OutRect.Top < OutRect.Bottom)
     then begin
      Canvas.CopyRect(OutRect,BMP.Canvas,Rect(OutRect.Left-ShiftX,OutRect.Top-ShiftY,OutRect.Right-ShiftX,OutRect.Bottom-ShiftY));
      if (Fragment = SelectedFragment)
       then begin
        Canvas.Pen.Width:=5;
        Canvas.Pen.Color:=clRed;
        Canvas.Brush.Style:=bsClear;
        with OutRect do begin
        Left:=Left-Canvas.Pen.Width;
        Top:=Top-Canvas.Pen.Width;
        Right:=Right+Canvas.Pen.Width;
        Bottom:=Bottom+Canvas.Pen.Width;
        end;
        Canvas.Rectangle(OutRect);
        end;
      end;
    end;
  //. next fragment
  Fragment:=Fragment.Next;
  end;
end;
PaintBox.Canvas.Draw(0,0,Project_BMP);
end;

procedure TfmImageDivider.sbChangeContainerColorClick(Sender: TObject);
begin
ColorDialog.Color:=Project.ContainerColor;
if (ColorDialog.Execute)
 then begin
  Project.ContainerColor:=ColorDialog.Color;
  Project_Paint;
  end;
end;

procedure TfmImageDivider.sbAddNewFragmentClick(Sender: TObject);
begin
OpenPictureDialog.InitialDir:=ExtractFilePath(Application.ExeName)+'\'+FragmentsFolder;
if (OpenPictureDialog.Execute())
 then begin
  SelectedFragment:=Project.AddNewFragment(OpenPictureDialog.FileName,-Project__Out_OfsX,-Project__Out_OfsY);
  Project_Paint;
  end;
end;

procedure TfmImageDivider.PaintBoxPaint(Sender: TObject);
begin
Project_Paint;
end;

procedure TfmImageDivider.PaintBoxMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
case Button of
mbLeft: begin
  MovingFragment:=Project.FragmentAtPoint(X-Project__Out_OfsX,Y-Project__Out_OfsY);
  SelectedFragment:=nil;
  end;
mbRight: begin
  flProjectMoving:=true;
  end;
end;
end;

procedure TfmImageDivider.PaintBoxMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
case Button of
mbLeft: begin
  SelectedFragment:=MovingFragment;
  MovingFragment:=nil;
  Project_Paint;
  end;
mbRight: begin
  flProjectMoving:=false;
  end;
end;
end;

procedure TfmImageDivider.PaintBoxMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
if (MovingFragment <> nil)
 then begin
  MovingFragment.OfsX:=MovingFragment.OfsX+(X-Mouse_LastPos.X);
  MovingFragment.OfsY:=MovingFragment.OfsY+(Y-Mouse_LastPos.Y);
  Project_Paint;
  end
 else
  if (flProjectMoving)
   then begin
    Project__Out_OfsX:=Project__Out_OfsX+(X-Mouse_LastPos.X);
    Project__Out_OfsY:=Project__Out_OfsY+(Y-Mouse_LastPos.Y);
    Project_Paint;
    end;
//.
Mouse_LastPos.X:=X;
Mouse_LastPos.Y:=Y;
end;

procedure TfmImageDivider.sbDivideImageClick(Sender: TObject);
begin
Project.DivideFragmentImageToSegments();
end;

procedure TfmImageDivider.sbDestroySelectedFragmentClick(Sender: TObject);
begin
if (SelectedFragment = nil) then Exit; //. ->
Project.DestroyFragment(SelectedFragment);
SelectedFragment:=nil;
Project_Paint;
end;

procedure TfmImageDivider.sbMoveUpSelectedFragmentClick(Sender: TObject);
begin
if (SelectedFragment = nil) then Exit; //. ->
Project.MoveUpFragment(SelectedFragment);
Project_Paint;
end;

procedure TfmImageDivider.sbMoveDownSelectedFragmentClick(Sender: TObject);
begin
if (SelectedFragment = nil) then Exit; //. ->
Project.MoveDownFragment(SelectedFragment);
Project_Paint;
end;

procedure TfmImageDivider.sbNewProjectClick(Sender: TObject);
begin
Project.Clear;
Project_Paint;
end;

end.
